*! version 3.1.2  03nov1999
program define ksm
	version 6
	syntax varlist(min=2 max=2 numeric) [if] [in] [, /*
		*/ Line Weight noGraph GEN(string) BWidth(real 0) /*
		*/ Adjust LOWess LOgit T1title(string) /*
		*/ Symbol(string) Connect(string) * ]
	if "`gen'"~="" {
		confirm new var `gen'
	}
	local tmpwgt "`weight'"
	local weight 
	marksample touse
	local weight "`tmpwgt'"
	local tmpwgt
	
	if "`lowess'"!="" {
		local weight "weight"
		local line "line"
	}
	tempvar Y X smooth
	tokenize `varlist'
	quietly {
		gen `Y'=`1' if `touse'
		gen `X'=`2' if `touse'
		summ `X'
		local cnt = r(N)
		if `cnt'<3 { noisily error 2001 }
		if r(max)-r(min)<1e-30 {
			di in red "Range of `2' is too small"
			exit 499
		}
		_crcslbl `Y' `1'
		_crcslbl `X' `2'
/*
	Set up complete.
*/
		local bw = `bwidth'
		if `bw' <= 0 | `bw' >= 1 { local bw .8 }
/*
	smopt=0 (unwtd mean), 1(unwtd line), 2(wtd mean), 3(wtd line).
*/
		local smopt = ("`line'"=="line")+cond("`weight'"=="weight",2,0)
		Ksm `Y' `X' `cnt' `bw' `smooth' `smopt' `touse'
/*
	Adjust smooth so that mean of Y-values equals mean of smoothed values.
*/
		if "`adjust'" == "adjust" {
			summ `smooth'
			local mean = r(mean)
			summ `Y'
			replace `smooth' = `smooth'*r(mean)/`mean' if `touse'
		}
		if "`logit'" == "logit" {
			local adj = 1/`cnt'
			local small = 0.0001
			replace `smooth' = `adj' if `smooth'<`small' & `touse'
			replace `smooth' = 1-`adj' if `smooth'>(1-`small') /*
						*/ & `touse'
			replace `smooth' = ln(`smooth'/(1-`smooth')) if `touse'
		}
	}
/*
	Graph (if required)
*/
	if "`graph'" == "" { 
		if `"`t1title'"' ==""{
			if "`lowess'"!="" {
				local t1title "Lowess smoother"
			}
			else if "`line'" == "line" {
				local t1title "Running line smoother"
			}
			else 	local t1title "Running mean smoother"
			local t1title `"`t1title', bandwidth = `bw'"'
		}
		if "`logit'"=="" {
			local symbol = cond("`symbol'"=="","oi","`symbol'")
			local connect = cond("`connect'"=="",".l","`connect'")
			graph `Y' `smooth' `X', `options' /*
				*/ t1(`"`t1title'"') /*
				*/ s(`symbol') c(`connect')
		}
		else {
			local symbol = cond("`symbol'"=="","i","`symbol'")
			local connect = cond("`connect'"=="","l","`connect'")
			gr `smooth' `X', `options' t1(`"`t1title'"') /*
				*/ s(`symbol') c(`connect')
		}
	}
	if "`gen'"~="" {
		rename `smooth' `gen'
	}
end


program define Ksm
	args Y X count bwidth Ys smopt touse
	tempvar W
	capture drop `Ys'
/*
	Args: 1=Y, 2=X, 3=count, 4=bandwidth, 5=smoothed Y,
	6=0 (unwtd mean), 1(unwtd line), 2(wtd mean), 3(wtd line).
	NOTE: bandwidth is expressed as a fraction of the sample size,
	not a number of sample values.
*/

	tempvar revuse 
	gen byte `revuse' = cond(`touse',1,2)
	sort `revuse' `X'
	drop `revuse'
	/* touse sample is now 1/`count' */

/*
	Using symmetric neighbourhoods.
	`k' is half-bandwidth. Initialise using points 1,...,k+1.
	Delta is distance between current Y and furthest neighbour in the
	interval. `W' is tricube weight function within the interval.
*/
	

	gen `W' = 1 in 1/`count'
	gen `Ys' = .
	local wtd = (`smopt'>1)
	local mean = (`smopt'==0 | `smopt'==2)
	local k = int((`bwidth'*`count'-0.5)/2)
	local i 0
	while `i'<`count' {
		local i = `i'+1
		local x0 = `X'[`i']
		local imk = max(1, `i'-`k')
		local ipk = min(`i'+`k', `count')
		if `wtd' {
			local delta = 1.0001* /*
				*/ max(`X'[`ipk']-`x0', `x0'-`X'[`imk'])
			replace `W' = cond(`delta'>1e-30, /*
				*/ (1-(abs(`X'-`x0')/`delta')^3)^3 , 1) /*
				*/ in `imk'/`ipk'
		}
		if `mean' {
			summ `Y' [aw=`W'] in `imk'/`ipk'
			replace `Ys' = r(mean) in `i'
		}
		else {
			reg `Y' `X' [aw=`W'] in `imk'/`ipk'
			replace `Ys' = _b[_cons]+_b[`X']*`x0' in `i'
		}
	}
end
