*! version 1.5.0  30Jun2000
* jw/vb - ics
program define tabstat, rclass
	version 6

	syntax varlist(numeric) [if] [in] [aw fw] [ , /*
	*/ 	BY(varname) CASEwise Columns(str) Format Format2(str) /*
	*/ 	LAbelwidth(int 16) LOngstub Missing /*
	*/ 	SAME SAVE noSEP Statistics(str) STATS(str) noTotal ]

	if "`casewise'" != "" {
		local same same
	}

	if `"`stats'"' != "" {
		if `"`statistics'"' != "" {
			di in red /*
			*/ "may not specify both statistics() and stats() options"
			exit 198
		}
		local statist `"`stats'"'
		local stats
	}

	if "`total'" != "" & "`by'" == "" {
		di in gr "nothing to display
		exit 0
	}

	if "`format'" != "" & `"`format2'"' != "" {
		di in re "may not specify both format and format()"
		exit 198
	}
	if `"`format2'"' != "" {
		capt local tmp : display `format2' 1
		if _rc {
			di in re `"invalid %format in format() : `format2'"'
			exit 120
		}
	}

	if `"`columns'"' == "" {
		local incol "variables"
	}
	else if `"`columns'"' == substr("variables",1,length(`"`columns'"')) {
		local incol "variables"
	}
	else if `"`columns'"' == substr("statistics",1,length(`"`columns'"')) {
		local incol "statistics"
	}
	else {
		di in red `"column(`columns') invalid -- specify "' /*
			*/ "column(variables) or column(statistics)"
		exit 198
	}

	if "`longstub'" != "" | "`by'" == "" {
		local descr descr
	}

	* sample selection

	marksample touse, novar
	if "`same'" != "" {
		markout `touse' `varlist'
	}
	if "`by'" != "" & "`missing'" == "" {
		markout `touse' `by' , strok
	}
	qui count if `touse'
	local ntouse = r(N)
	if `ntouse' == 0 {
		error 2000
	}
	if `"`weight'"' != "" {
		local wght `"[`weight'`exp']"'
	}

	* varlist -> var1, var2, ... variables
	*            fmt1, fmt2, ... display formats

	tokenize "`varlist'"
	local nvars : word count `varlist'
	local i 1
	while `i' <= `nvars' {
		local var`i' ``i''
		if "`format'" != "" {
			local fmt`i' : format ``i''
		}
		else if `"`format2'"' != "" {
			local fmt`i' `format2'
		}
		else	local fmt`i' %9.0g
		local i = `i' + 1
	}
	if `nvars' == 1 & `"`columns'"' == "" {
		local incol statistics
	}

	* Statistics

	Stats `statistics'
	local stats   `r(names)'
	local expr    `r(expr)'
	local summopt `r(summopt)'
	local nstats : word count `stats'

	tokenize `expr'
	local i 1
	while `i' <= `nstats' {
		local expr`i' ``i''
		local i = `i' + 1
	}
	tokenize `stats'
	local i 1
	while `i' <= `nstats' {
		local name`i' ``i''
		local names "`names' ``i''"
		if `i' < `nstats' { local names "`names'," }
		local i = `i' + 1
	}
	if "`sep'" == "" & ( (`nstats' > 1 & "`incol'" == "variables") /*
		 */              |(`nvars' > 1  & "`incol'" == "statistics")) {
		local sepline yes
	}

	local matsize : set matsize
	local matreq = max(`nstats',`nvars')
	if `matsize' < `matreq' {
		di in re "set matsize to at least `matreq' (see help matsize for details)"
		exit 908
	}

	* compute the statistics
	* ----------------------

	if "`by'" != "" {
		* conditional statistics are saved in matrices Stat1, Stat2, etc

		* the data are sorted on by groups, putting unused obs last
		* be careful not to change the sort order
		* note that touse is coded -1/0 rather than 1/0!

		qui replace `touse' = - `touse'
		sort `touse' `by'

		local bytype : type `by'
		local by2 0
		local iby 1
		while `by2' < `ntouse'  {
			tempname Stat`iby'
			mat `Stat`iby'' = J(`nstats',`nvars',0)
			mat colnames `Stat`iby'' = `varlist'
			mat rownames `Stat`iby'' = `stats'

			* range `iby1'/`iby2' refer to obs in the current by-group
			local by1 = `by2' + 1
			qui count if (`by'==`by'[`by1']) & (`touse')
			local by2 = `by1' + r(N) - 1

			* loop over all variables
			local i 1
			while `i' <= `nvars' {
				qui summ `var`i'' in `by1'/`by2' `wght', `summopt'
				local is 1
				while `is' <= `nstats' {
					* set matrix[is,i] with mv-handling
					SetMat  `Stat`iby''[`is',`i']   `expr`is''
					local is = `is' + 1
				}
				local i = `i' + 1
			}

			* save label for groups in lab1, lab2 etc
			if substr("`bytype'",1,3) != "str" {
				local iby1 = `by'[`by1']
				local lab`iby' : label (`by') `iby1'
			}
			else	local lab`iby' = `by'[`by1']

			local iby = `iby' + 1
		}
		local nby = `iby' - 1

		/* wwg did not like this. Fine.
		if `nby' == 1 {
			di in gr "(`by' is constant on selected sample)"
			local by
			local nby 0
		}
		*/
	}
	else	local nby 0

	if "`total'" == "" {
		* unconditional (Total) statistics are stored in Stat`nby+1'
		local iby = `nby'+1

		tempname Stat`iby'
		mat `Stat`iby'' = J(`nstats',`nvars',0)
		mat colnames `Stat`iby'' = `varlist'
		mat rownames `Stat`iby'' = `stats'

		local i 1
		while `i' <= `nvars' {
			qui summ `var`i'' if `touse' `wght' , `summopt'
			local is 1
			while `is' <= `nstats' {
				* set matrix[is,i] with mv-handling
				SetMat  `Stat`iby''[`is',`i']  `expr`is''
				local is = `is' + 1
			}
			local i = `i' + 1
		}
		local lab`iby' "Total"
	}

	* display results
	* ---------------

	* constants for displaying results

	local labw = min(max(`labelwidth',8),32)
	if "`by'" != "" {
		if substr("`bytype'",1,3) != "str" {
			local lv : value label `by'
			if "`lv'" != "" {
				local lg : label (`by') maxlength
				local wby = min(`labw',`lg')
			}
			else	local wby 8
		}
		else {
			local wby = min(real(substr("`bytype'",4,.)),`labw')
			local bytype str
		}
		local wby = max(length("`by'"), `wby')
	}
	else	local wby 8

	local lleft = (1 + `wby')*("`by'"!="") + 9*("`descr'"!="")
	local cbar  = `lleft' + 1

	local lsize : set display linesize
	* number of non-label elements in the row of a block
	local neblock = int((`lsize' - `cbar')/10)
	* number of blocks if stats horizontal
	local nsblock  = 1 + int((`nstats'-1)/`neblock')
	* number of blocks if variables horizontal
	local nvblock  = 1 + int((`nvars'-1)/`neblock')

	* left align by-label if also descr
	if "`descr'" != "" & "`by'" != "" {
		local aby "-"
	}

	if "`incol'" == "statistics" {

		* display the results: horizontal = statistics (block wise)
		* ---------------------------------------------------------

		* header

		if "`descr'" == "" {
			di in gr _n `"Summary for variables: `varlist'"'
			if "`by'" != "" {
				local labelby : var label `by'
				if `"`labelby'"' != "" {	local labelby `"(`labelby')"'	}
				di _col(6) in gr `"by categories of: `by' `labelby'"'
			}
		}
		di

		* loop over all nsblock blocks of statistics

		local isblock 1
		local is2 0
		while `isblock' <= `nsblock' {

			* is1..is2 are indices of statistics in a block
			local is1 = `is2' + 1
			local is2 = min(`nstats', `is1'+`neblock'-1)

			* display header
			if "`by'" != "" { di in gr %`aby'`wby's "`by'" " " _c }
			if "`descr'" != "" { di in gr "variable " _c	}
			di in gr "|" _c
			local is `is1'
			while `is' <= `is2' {
				di in gr %10s "`name`is''" _c
				local is = `is' + 1
			}
			local ndash = 10*(`is2'-`is1'+1)
			di _n in gr _dup(`lleft') "-" "+" _dup(`ndash') "-"

			* loop over the categories of -by- (1..nby) and -total- (nby+1)
			local nbyt = `nby' + ("`total'"=="")
			local iby 1
			while `iby' <= `nbyt'{
				local i 1
				while `i' <= `nvars' {
					if "`by'" != "" {
						if `i' == 1 {
							local lab = substr(`"`lab`iby''"'0, 1, `wby')
							di in gr %`aby'`wby's `"`lab'"' " " _c
						}
						else	di _skip(`wby') " " _c
					}
					if "`descr'" != "" {
						di in gr %8s "`var`i''" " " _c
					}
					di in gr "|" _c

					local is `is1'
					while `is' <= `is2' {
						GetMat s : `fmt`i'' `Stat`iby''[`is',`i']
						di in ye %10s "`s'" _c
						local is = `is' + 1
					}
					di
					local i = `i' + 1
				}
				local iby = `iby' + 1

				if ("`sepline'"!="") | (`iby'>`nbyt') | /*
					*/ ((`iby'==`nbyt') & ("`total'"=="")) {
					di in gr _dup(`lleft') "-" "+" _dup(`ndash') "-"
				}
			}

			local isblock = `isblock' + 1
			if `isblock' <= `nsblock' {
				display
			}
		} /* isblock */
	}

	else {

		* display the results: horizontal = variables (block wise)
		* --------------------------------------------------------

		* header

		if "`descr'" == "" {
			di in gr _n `"Summary statistics:`names'"'
			if "`by'" != "" {
				local labelby : var label `by'
				if `"`labelby'"' != "" {	local labelby `"(`labelby')"'	}
				di in gr `"  by categories of: `by' `labelby'"'
			}
		}
		di

		* loop over all nvblock blocks of variables

		local iblock 1
		local i2 0
		while `iblock' <= `nvblock' {

			* i1..i2 are indices of variables in a block
			local i1 = `i2' + 1
			local i2 = min(`nvars', `i1'+`neblock'-1)

			* display header
			if "`by'" != "" { di in gr %`aby'`wby's "`by'" " " _c }
			if "`descr'" != "" { di in gr "   stats " _c	}
			di in gr "|" _c
			local i `i1'
			while `i' <= `i2' {
				di in gr %10s "`var`i''" _c
				local i = `i' + 1
			}
			local ndash = 10*(`i2'-`i1'+1)
			di _n in gr _dup(`lleft') "-" "+" _dup(`ndash') "-"

			* loop over the categories of -by- (1..nby) and -total- (nby+1)
			local nbyt = `nby' + ("`total'"=="")
			local iby 1
			while `iby' <= `nbyt'{
				local is 1
				while `is' <= `nstats' {
					if "`by'" != "" {
						if `is' == 1 {
							local lab = substr(`"`lab`iby''"'0, 1, `wby')
							di in gr %`aby'`wby's `"`lab'"' " " _c
						}
						else	di _skip(`wby') " " _c
					}
					if "`descr'" != "" {
						di in gr %8s "`name`is''" " " _c
					}
					di in gr "|" _c

					local i `i1'
					while `i' <= `i2' {
						GetMat s : `fmt`i'' `Stat`iby''[`is',`i']
						di in ye %10s "`s'" _c
						local i = `i' + 1
					}
					di
					local is = `is' + 1
				}
				local iby = `iby' + 1

				if ("`sepline'"!="") | (`iby'>`nbyt') | /*
					*/ ((`iby'==`nbyt') & ("`total'"=="")) {
					di in gr _dup(`lleft') "-" "+" _dup(`ndash') "-"
				}
			}

			local iblock = `iblock' + 1
			if `iblock' <= `nvblock' {
				display
			}
		} /* iblock */
	}

	* save results (mainly for certification)
	* ---------------------------------------

	if "`save'" != "" {
		if "`total'" == "" {
			local iby = `nby'+1
			return matrix StatTot `Stat`iby''
		}

		if "`by'" == "" { exit }
		local iby 1
		while `iby' <= `nby' {
			return matrix Stat`iby' `Stat`iby''
			return local  name`iby' `"`lab`iby''"'
			local iby = `iby' + 1
		}
	}
end


* As a work around that matrices can't have missing values,
* mv's are coded as the magic number 1e+300

program define SetMat
	matrix `1' = cond(`2' != ., `2', 1e+300)
end

program define GetMat
	args rslt colon fmt exp
	local s : display `fmt' =cond(`exp' != 1e+300, `exp', .)
	c_local `rslt'  `s'
end


/* Stats str
   processes the contents() option. It returns in
     r(names)   -- names of statistics, separated by blanks
     r(expr)    -- r() expressions for statistics, separated by blanks
     r(summopt) -- option for summarize command (meanonly, detail)
*/
program define Stats, rclass
	if `"`0'"' == "" {
		local 0 "mean"
	}

	* ensure that order of requested statistics is preserved
	* invoke syntax for each word in input
	local class 0
	tokenize `0'
	while "`1'" != "" {
		local 0 = lower(`", `1'"')
		syntax [, n MEan sd SUm COunt MIn MAx Range SKewness Kurtosis /*
			*/  SDMean p1 p5 p10 p25 p50 p75 p90 p95 p99 iqr q MEDian ]

		if "`median'" != "" {
			local p50 p50
		}
		if "`count'" != "" {
			local n n
		}

		* class 1 : available via -summarize, meanonly-

		* summarize.r(N) returns #obs (note capitalization)
		if "`n'" != "" {
			local n N
		}
		local s "`n'`min'`mean'`max'`sum'"
		if "`s'" != "" {
			local names "`names' `s'"
			local expr  "`expr' r(`s')"
			local class = max(`class',1)
		}
		if "`range'" != "" {
			local names "`names' range"
			local expr  "`expr' r(max)-r(min)"
			local class = max(`class',1)
		}

		* class 2 : available via -summarize-

		if "`sd'" != "" {
			local names "`names' sd"
			local expr  "`expr' r(sd)"
			local class = max(`class',2)
		}
		if "`sdmean'" != "" {
			local names "`names' sd(mean)"
			local expr  "`expr' r(sd)/sqrt(r(N))"
			local class = max(`class',2)
		}

		* class 3 : available via -detail-

		local s "`skewness'`kurtosis'`p1'`p5'`p10'`p25'`p50'`p75'`p90'`p95'`p99'"
		if "`s'" != "" {
			local names "`names' `s'"
			local expr  "`expr' r(`s')"
			local class = max(`class',3)
		}
		if "`iqr'" != "" {
			local names "`names' iqr"
			local expr  "`expr' r(p75)-r(p25)"
			local class = max(`class',3)
		}
		if "`q'" != "" {
			local names "`names' p25 p50 p75"
			local expr  "`expr' r(p25) r(p50) r(p75)"
			local class = max(`class',3)
		}

		mac shift
	}

	return local names `names'
	return local expr  `expr'
	if `class' == 1 {
		return local summopt "meanonly"
	}
	else if `class' == 3 {
		return local summopt "detail"
	}
end
exit


Some notes on implementation
----------------------------

* as matrices can't contain missing values, we have to use a magic number
  scheme. Clearly this breaks down if the magic number occurs as a regular
  value. Currently, we do not test for this.

* the code that displays the tables assumes that the names of statistics and
  variables are at most 8 chars

* the code for -columns(statistics)- and -columns(variables)-
  could of course be unified by using matrix transposes, and extracting
  names/labels from the row/column labels. Should I do this?


History of changes since release (29jun2000)
--------------------------------------------

1.5.0  minor additions and bug fixes
       - added sdmean to list of statistics (request njc)
       - made statistics() case-insensitive (request njc)
       - changed the default of columns is only one var is specified
       - ensured that a sepline is displayed by default only if
         a cell takes up more than one row
       - fixed bug in error message for invalid columns()

1.4.2  official release
